@@ -54,6 +54,19 @@ class AutoConvHandler:
 		query.edit_message_text(f"{new_state}",reply_markup=keyboard,parse_mode=new_state.mode)
 		return self.NEXT
 
+	def _build_dynamic_stuff(self,state):
+		ret = state.action(self.update,self.context) if state.action else None
+		if state.routes:
+			ro,de,ba = state.routes(self.update,self.context)
+			self.conversation.add_routes(state,ro,de,ba)
+		if state.build: 
+			keyboard = state.build(self.update,self.context)
+			keyboard,size = keyboard if isinstance(keyboard,tuple) else (keyboard,None)
+			state.add_keyboard(keyboard,size,max_row=state.max_row)
+		keyboard = self._build_keyboard(state)
+		reply_msg = state.msg if ret == None else state.msg.replace('@@@',ret)
+		return keyboard,reply_msg
+
 	def restart(self):
 		if self.update:
 			telegram_id = self.update.effective_chat.id
@@ -74,10 +87,7 @@ class AutoConvHandler:
 			if delete_first: update.message.delete()
 			state = self.conversation.start
 			self.context.user_data.update({telegram_id:{'state':state,'error':False,'data':{}}})
-			if state.build: state.add_keyboard(state.build(self.update,self.context),max_row=state.max_row)
-			keyboard = self._build_keyboard(state)
-			ret = state.action(self.update,self.context) if state.action else None
-			reply_msg = state.msg if ret == None else state.msg.replace('@@@',ret)
+			keyboard,reply_msg = self._build_dynamic_stuff(state)
 			msg = self.update.message.reply_text(f'{reply_msg}',reply_markup=keyboard,parse_mode=state.mode)
 			self.context.user_data.get(telegram_id).update({'bot-msg':msg})
 			return self.NEXT
@@ -95,13 +105,7 @@ class AutoConvHandler:
 		# next stage
 		typed_data = state.data_type(data) if data != 'BACK' else 'BACK'
 		state = self._change_state(telegram_id,typed_data)
-		ret = state.action(self.update,self.context) if state.action else None
-		if state.routes:
-			ro,de,ba = state.routes(self.update,self.context)
-			self.conversation.add_routes(state,ro,de,ba)
-		msg = state.msg if ret == None else state.msg.replace('@@@',ret)
-		if state.build: state.add_keyboard(state.build(self.update,self.context),max_row=state.max_row)
-		keyboard = self._build_keyboard(state)
-		to_reply(f'{msg}',reply_markup=keyboard,parse_mode=state.mode,disable_web_page_preview=not state.webpage_preview)
+		keyboard,reply_msg = self._build_dynamic_stuff(state)
+		to_reply(f'{reply_msg}',reply_markup=keyboard,parse_mode=state.mode,disable_web_page_preview=not state.webpage_preview)
 		if state == self.conversation.end: context.user_data.update({telegram_id:None}); return ConversationHandler.END
 		return self.NEXT
\ No newline at end of file
