@@ -18,6 +18,7 @@ class State:
 		self.build = None
 		self.back = back
 		self.webpage_preview = webpage_preview
+		self.routes = None
 	
 	def __str__(self):
 		return f'State <{self.name}>'
@@ -49,4 +50,9 @@ class State:
 		raise_type_error(function,'function',(Callable))
 		raise_type_error(max_row,'max_row',(int))
 		self.build = function
-		self.max_row = max_row
\ No newline at end of file
+		self.max_row = max_row
+
+	def add_dynamic_routes(self,function):
+		'''Add function to create dynamic routes | must return 3 value (routes,default,back)'''
+		raise_type_error(function,'function',(Callable))
+		self.routes = function
